/* vi:set wm=0 ai tabstop=4 shiftwidth=4: */

/*
 * sctl - Bearcat scanner control program
 * Copyright (C) 2001 Ian Johnston <reaper@muppetlabs.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <ctype.h>
#include <signal.h>

#include "config.h"
#include "serial.h"
#include "commands.h"
#include "human_commands.h"
#include "usage.h"
#include "utils.h"
#include "main.h"

int fd; /* because we need to have the fd available for the signal handler */
int scannermodel = DEFAULT_MODEL; /* scanner model id... used throughout the program to
								   * determine how certain commands are given to the scanner */

/***********************************************************************
 * the main function.  ain't it grand?
 ***********************************************************************/
int main(int argc, char **argv)
{
	int i, cmd_argc, cmd_loc, cmd_arg_loc;
	char **cmd_argv;
	char serial_port[20];
	char *configloc;
	char configline[81];
	FILE *configfile;
	char errorstr[100];
	
	signal(SIGINT, sighandler);

	bzero(serial_port, 20);

	cmd_loc = 1;
	cmd_arg_loc = 2;
	strncat(serial_port, DEFAULT_PORT, 20);  /* default port should be ok for most users, imo. */

	/* check ~/.sctl for configuration information */

	configloc = strcat(getenv("HOME"), "/.sctl");
	if ((configfile = fopen(configloc, "r")) == NULL)
	{
		if (errno == ENOENT)	/* config file not found -- create it */
		{
			if ((configfile = fopen(configloc, "w")) != NULL)
			{
				fprintf(configfile, "# sctl configuration file\n# lines starting with # are ignored\n");
				fprintf(configfile, "port /dev/ttyS0\n");
				fprintf(configfile, "# model can be bc895xlt bc245xlt bc780xlt pro2052 or unknown - *CASE SENSITIVE*\n");
				fprintf(configfile, "model unknown\n");
			}
			else				/* unable to write a config file. if they don't have write permission on their home directory or something else weird */
			{
				configloc = strcat("write to ", configloc);
				perror(configloc);
			}
		}
		else					/* some other problem with the file, ie permissions */
		{
			perror(configloc);
		}
	}
	else						/* config file ok -- read it in */
	{
		while (fgets(configline, 81, configfile) != NULL)
		{
			char configcmd[81], configvar[81];

			if (configline[0] == '#' || configline[0] == '\n' || configline[0] == ' ')
			{
				/* ignore comments and blank lines */
				continue;
			}
			else
			{
				sscanf(configline, "%s %s", configcmd, configvar);
				if (strcmp(configcmd, "port") == 0)
				{
					bzero(serial_port, 20);
					strncat(serial_port, configvar, 20);
				}
				else if (strcmp(configcmd, "model") == 0)
				{
					if (strcmp(configvar, "bc895xlt") == 0)
						scannermodel = BC895XLT;
					else if (strcmp(configvar, "bc245xlt") == 0)
						scannermodel = BC245XLT;
					else if (strcmp(configvar, "bc780xlt") == 0)
						scannermodel = BC780XLT;
					else if (strcmp(configvar, "pro2052") == 0)
						scannermodel = PRO2052;
					else
					{
						snprintf(errorstr, 100, "%s: Scanner model %s is not supported!", configloc, configvar);
						complain(errorstr);
						scannermodel = MODELUNKNOWN;
					}
				}
				else
				{
					snprintf(errorstr, 100, "%s: Ignoring unknown configuration command: %s", configloc, configcmd);
					complain(errorstr);
				}
			}
		}
	}

	/* process global options */

	for (i=1; i<argc; i++)
	{
		if (strcmp(argv[i], "-p") == 0)			/* port */
		{
			bzero(serial_port, 20);
			strncat(serial_port, argv[++i], 20);
			cmd_loc += 2;
			cmd_arg_loc += 2;
		}
		else if (strcmp(argv[i], "-m") == 0)	/* model */
		{
			char mdl[9], *p;
			
			strcpy(mdl, argv[++i]);
			for (p = mdl; *p != '\0'; p++)
				*p = tolower(*p);
			if (strcmp(mdl, "bc895xlt") == 0)
				scannermodel = BC895XLT;
			else if (strcmp(mdl, "bc245xlt") == 0)
				scannermodel = BC245XLT;
			else if (strcmp(mdl, "bc780xlt") == 0)
				scannermodel = BC780XLT;
			else if (strcmp(mdl, "pro2052") == 0)
				scannermodel = PRO2052;
			else
				scannermodel = MODELUNKNOWN; /* error will be caught below */

			cmd_loc += 2;
			cmd_arg_loc += 2;
		}
	}

	/* now set our variables that can be passed in to human functions */

	cmd_argv = malloc(sizeof(char**) * (argc - cmd_arg_loc));
	cmd_argc = argc - cmd_arg_loc;

	for (i=cmd_arg_loc; i<argc; i++)
	{
		cmd_argv[i-cmd_arg_loc] = argv[i];
	}
	
	fd = open_port(serial_port);
	setup_port(fd);

	/* no valid ~/.sctl?  no -m command line option?  no define default_scanner? tell them this. */
	if (scannermodel == MODELUNKNOWN)
	{
		complain("Unknown scanner model.  Assuming scanner is BC895XLT.\n\tPlease change ~/.sctl or use -m <model> option if this is incorrect.");
		scannermodel = BC895XLT;
	}

	if (argv[cmd_loc] == NULL)
	{
		usage(C_NONE);
		return(1);
	}

	switch (get_cmd(argv[cmd_loc]))
	{
		case C_ATT:
			human_do_att(fd, cmd_argc, cmd_argv);
			break;

		case C_DATA:
			human_do_data(fd, cmd_argc, cmd_argv);
			break;

		case C_DELAY:
			human_do_delay(fd, cmd_argc, cmd_argv);
			break;

		case C_DUMP:
			human_do_dump(fd, cmd_argc, cmd_argv);
			break;

		case C_HELP:
			human_help(fd, cmd_argc, cmd_argv);
			break;

		case C_ID_REPORT:
			human_do_id_report(fd, cmd_argc, cmd_argv);
			break;

		case C_LIGHT:
			human_do_light(fd, cmd_argc, cmd_argv);
			break;

		case C_LOAD:
			human_do_load(fd, cmd_argc, cmd_argv);
			break;

		case C_LOCKOUT:
			human_do_lockout(fd, cmd_argc, cmd_argv);
			break;

		case C_MEM:
			human_do_mem(fd, cmd_argc, cmd_argv);
			break;

		case C_MUTE:
			human_do_mute(fd, cmd_argc, cmd_argv);
			break;

		case C_SCAN:
			human_do_scan(fd, cmd_argc, cmd_argv);
			break;

		case C_SCAN_REPORT:
			human_do_scan_report(fd, cmd_argc, cmd_argv);
			break;

		case C_SEARCH:
			human_do_search(fd, cmd_argc, cmd_argv);
			break;

		case C_STATUS:
			human_do_status(fd, cmd_argc, cmd_argv);
			break;

		case C_TALKGROUP:
			human_do_talkgroup(fd, cmd_argc, cmd_argv);
			break;

		case C_VERSION:
			human_do_version(fd, cmd_argc, cmd_argv);
			break;

		default: /* invalid command specified... */
			usage(C_NONE);
			break;
	}

	exit(EXIT_SUCCESS);
}


/***********************************************************************
 * figure out what command they want to run
 ***********************************************************************/
int get_cmd(char *cmd)
{
	/* strncmp is used to get as many chars as necessary to uniquely
	 * identify the command -- the the user can say "me", "mem", or
	 * "memory" equally well. */

	if (strncmp(cmd, "a", 1) == 0)
		return(C_ATT);
	else if (strncmp(cmd, "da", 2) == 0)
		return(C_DATA);
	else if (strncmp(cmd, "de", 2) == 0)
		return(C_DELAY);
	else if (strncmp(cmd, "du", 2) == 0)
		return(C_DUMP);
	else if (strncmp(cmd, "h", 1) == 0)
		return(C_HELP);
	else if (strncmp(cmd, "i", 1) == 0)
		return(C_ID_REPORT);
	else if (strncmp(cmd, "lig", 3) == 0)
		return(C_LIGHT);
	else if (strncmp(cmd, "loa", 3) == 0)
		return(C_LOAD);
	else if (strncmp(cmd, "loc", 3) == 0)
		return(C_LOCKOUT);
	else if (strncmp(cmd, "me", 2) == 0)
		return(C_MEM);
	else if (strncmp(cmd, "mu", 2) == 0)
		return(C_MUTE);
	else if (strncmp(cmd, "sc", 2) == 0)
		return(C_SCAN);
	else if (strncmp(cmd, "sr", 2) == 0)
		return(C_SCAN_REPORT);
	else if (strncmp(cmd, "se", 2) == 0)
		return(C_SEARCH);
	else if (strncmp(cmd, "st", 2) == 0)
		return(C_STATUS);
	else if (strncmp(cmd, "t", 1) == 0)
		return(C_TALKGROUP);
	else if (strncmp(cmd, "v", 1) == 0)
		return(C_VERSION);

	/* if we get this far, something's wrong */
	return(-1);
}


/***********************************************************************
 * a signal handler to reset the scanner's mode, useful mostly for
 id_report() at this point.
 ***********************************************************************/
void sighandler(int sig)
{
	char answer[100];
	int exitcode;

	signal(sig, SIG_IGN);

	bzero(answer, 100);
	do_command(fd, "IDF", answer);

	if (strncmp(answer, "OK", 2) == 0)
	{
		puts("ID report mode disabled\n");

		exitcode = 0;
	}
	else
	{
		puts("Unable to disable ID report mode\n");
		exitcode = 1;
	}

	bzero(answer, 100);
	do_command(fd, "QUF", answer);

	if (strncmp(answer, "OK", 2) == 0)
	{
		puts("Scan report mode disabled\n");
	}
	else
	{
		puts("Unable to disable scan report mode\n");
		exitcode = 1;
	}

	exit(exitcode);
}
